@@ -39,9 +39,8 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_LOGGING_CATEGORY(lcEv, "qt.quick3d.event")
-Q_LOGGING_CATEGORY(lcPick, "qt.quick3d.pick")
-Q_LOGGING_CATEGORY(lcHover, "qt.quick3d.hover")
+Q_STATIC_LOGGING_CATEGORY(lcEv, "qt.quick3d.event")
+Q_STATIC_LOGGING_CATEGORY(lcPick, "qt.quick3d.pick")
 
 static bool isforceInputHandlingSet()
 {
@@ -189,7 +188,7 @@ public:
 
 /*!
     \qmltype View3D
-    \inherits QQuickItem
+    \inherits Item
     \inqmlmodule QtQuick3D
     \brief Provides a viewport on which to render a 3D scene.
 
@@ -247,7 +246,6 @@ QQuick3DViewport::QQuick3DViewport(QQuickItem *parent)
     setFlag(ItemHasContents);
     m_camera = nullptr;
     m_sceneRoot = new QQuick3DSceneRootNode(this);
-    m_environment = new QQuick3DSceneEnvironment(m_sceneRoot);
     m_renderStats = new QQuick3DRenderStats();
     QQuick3DSceneManager *sceneManager = new QQuick3DSceneManager();
     QQuick3DObjectPrivate::get(m_sceneRoot)->refSceneManager(*sceneManager);
@@ -293,6 +291,8 @@ QQuick3DViewport::~QQuick3DViewport()
     delete m_sceneRoot;
     m_sceneRoot = nullptr;
 
+    delete m_builtInEnvironment;
+
     // m_renderStats is tightly coupled with the render thread, so can't delete while we
     // might still be rendering.
     m_renderStats->deleteLater();
@@ -370,10 +370,26 @@ QQuick3DCamera *QQuick3DViewport::camera() const
 
     This property specifies the SceneEnvironment used to render the scene.
 
+    \note Setting this property to \c null will reset the SceneEnvironment to the default.
+
     \sa SceneEnvironment
 */
 QQuick3DSceneEnvironment *QQuick3DViewport::environment() const
 {
+    if (!m_environment) {
+        if (!m_builtInEnvironment) {
+            m_builtInEnvironment = new QQuick3DSceneEnvironment;
+            // Check that we are on the "correct" thread, and move the environment to the
+            // correct thread if not. This can happen when environment() is called from the
+            // sync and no scene environment has been set.
+            if (QThread::currentThread() != m_sceneRoot->thread())
+                m_builtInEnvironment->moveToThread(m_sceneRoot->thread());
+            m_builtInEnvironment->setParentItem(m_sceneRoot);
+        }
+
+        return m_builtInEnvironment;
+    }
+
     return m_environment;
 }
 
@@ -755,6 +771,9 @@ void QQuick3DViewport::setEnvironment(QQuick3DSceneEnvironment *environment)
     m_environment = environment;
     if (m_environment && !m_environment->parentItem())
         m_environment->setParentItem(m_sceneRoot);
+
+    QQuick3DObjectPrivate::attachWatcherPriv(m_sceneRoot, this, &QQuick3DViewport::setEnvironment, environment, m_environment);
+
     emit environmentChanged();
     update();
 }
@@ -781,37 +800,8 @@ void QQuick3DViewport::setImportScene(QQuick3DNode *inScene)
     }
 
     m_importScene = inScene;
-    if (m_importScene) {
-        auto privateObject = QQuick3DObjectPrivate::get(m_importScene);
-        if (!privateObject->sceneManager) {
-            // If object doesn't already have scene manager, check from its children
-            QQuick3DSceneManager *manager = findChildSceneManager(m_importScene);
-            // If still not found, use the one from the scene root (scenes defined outside of an view3d)
-            if (!manager)
-                manager = QQuick3DObjectPrivate::get(m_sceneRoot)->sceneManager;
-            if (manager) {
-                manager->setWindow(window());
-                privateObject->refSceneManager(*manager);
-            }
-            // At this point some manager will exist
-            Q_ASSERT(privateObject->sceneManager);
-        }
-
-        connect(privateObject->sceneManager, &QQuick3DSceneManager::needsUpdate,
-                this, &QQuickItem::update);
-
-        QQuick3DNode *scene = inScene;
-        while (scene) {
-            QQuick3DSceneRootNode *rn = qobject_cast<QQuick3DSceneRootNode *>(scene);
-            scene = rn ? rn->view3D()->importScene() : nullptr;
-
-            if (scene) {
-                connect(QQuick3DObjectPrivate::get(scene)->sceneManager,
-                        &QQuick3DSceneManager::needsUpdate,
-                        this, &QQuickItem::update);
-            }
-        }
-    }
+    if (m_importScene)
+        updateSceneManagerForImportScene();
 
     emit importSceneChanged();
     update();
@@ -1228,7 +1218,7 @@ QList<QQuick3DPickResult> QQuick3DViewport::rayPickAll(const QVector3D &origin,
     return processedResultList;
 }
 
-void QQuick3DViewport::processPointerEventFromRay(const QVector3D &origin, const QVector3D &direction, QPointerEvent *event)
+void QQuick3DViewport::processPointerEventFromRay(const QVector3D &origin, const QVector3D &direction, QPointerEvent *event) const
 {
     internalPick(event, origin, direction);
 }
@@ -1854,7 +1844,7 @@ bool QQuick3DViewport::singlePointPick(QSinglePointEvent *event, const QVector3D
     for (const auto &pickResult : pickResults) {
         auto [item, position] = getItemAndPosition(pickResult);
         if (!item)
-            continue;
+            break;
         if (item == m_prevMouseItem && (position - m_prevMousePos).manhattanLength() < jitterLimit && !event->button()) {
             withinJitterLimit = true;
             break;
@@ -2075,4 +2065,69 @@ void QQuick3DViewport::rebuildExtensionList()
     update();
 }
 
+/*!
+    \internal
+
+    Private constructor for the QQuick3DViewport class so we can differentiate between
+    a regular QQuick3DViewport and one created for a specific usage, like XR.
+ */
+QQuick3DViewport::QQuick3DViewport(PrivateInstanceType type, QQuickItem *parent)
+    : QQuick3DViewport(parent)
+{
+    m_isXrViewInstance = type == PrivateInstanceType::XrViewInstance;
+}
+
+void QQuick3DViewport::updateCameraForLayer(const QQuick3DViewport &view3D, QSSGRenderLayer &layerNode)
+{
+    layerNode.explicitCameras.clear();
+    if (!view3D.m_multiViewCameras.isEmpty()) {
+        for (QQuick3DCamera *camera : std::as_const(view3D.m_multiViewCameras))
+            layerNode.explicitCameras.append(static_cast<QSSGRenderCamera *>(QQuick3DObjectPrivate::get(camera)->spatialNode));
+    } else if (view3D.camera()) {
+        layerNode.explicitCameras.append(static_cast<QSSGRenderCamera *>(QQuick3DObjectPrivate::get(view3D.camera())->spatialNode));
+    }
+}
+
+void QQuick3DViewport::updateSceneManagerForImportScene()
+{
+    auto privateObject = QQuick3DObjectPrivate::get(m_importScene);
+    if (!privateObject->sceneManager) {
+        // If object doesn't already have scene manager, check from its children
+        QQuick3DSceneManager *manager = findChildSceneManager(m_importScene);
+        // If still not found, use the one from the scene root (scenes defined outside of an view3d)
+        if (!manager)
+            manager = QQuick3DObjectPrivate::get(m_sceneRoot)->sceneManager;
+        if (manager) {
+            manager->setWindow(window());
+            privateObject->refSceneManager(*manager);
+        }
+
+        // In the case m_sceneRoot doesn't have a valid sceneManager,
+        // it means the view3d is not valid now.
+        if (!privateObject->sceneManager)
+            return;
+
+    }
+    connect(privateObject->sceneManager, &QQuick3DSceneManager::needsUpdate,
+            this, &QQuickItem::update, Qt::UniqueConnection);
+    connect(privateObject->sceneManager, &QObject::destroyed,
+            this, [&](QObject *) {
+                auto privateObject = QQuick3DObjectPrivate::get(m_importScene);
+                privateObject->sceneManager = nullptr;
+                updateSceneManagerForImportScene();
+            }, Qt::DirectConnection);
+
+    QQuick3DNode *scene = m_importScene;
+    while (scene) {
+        QQuick3DSceneRootNode *rn = qobject_cast<QQuick3DSceneRootNode *>(scene);
+        scene = rn ? rn->view3D()->importScene() : nullptr;
+
+        if (scene) {
+            connect(QQuick3DObjectPrivate::get(scene)->sceneManager,
+                    &QQuick3DSceneManager::needsUpdate,
+                    this, &QQuickItem::update, Qt::UniqueConnection);
+        }
+    }
+}
+
 QT_END_NAMESPACE
