@@ -249,7 +249,6 @@ QSSGRenderImageTexture QSSGBufferManager::loadRenderImage(const QSSGRenderImage
         if (it != renderExtensionTexture.end()) {
             it->usageCounts[currentLayer]++;
             result = it->renderImageTexture;
-            increaseMemoryStat(result.m_texture);
         }
     }
     return result;
@@ -391,32 +390,46 @@ QRhiTexture::Format QSSGBufferManager::toRhiFormat(const QSSGRenderTextureFormat
     case QSSGRenderTextureFormat::RGBA8_ETC2_EAC:
         return QRhiTexture::ETC2_RGBA8;
     case QSSGRenderTextureFormat::RGBA_ASTC_4x4:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_4x4:
         return QRhiTexture::ASTC_4x4;
     case QSSGRenderTextureFormat::RGBA_ASTC_5x4:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_5x4:
         return QRhiTexture::ASTC_5x4;
     case QSSGRenderTextureFormat::RGBA_ASTC_5x5:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_5x5:
         return QRhiTexture::ASTC_5x5;
     case QSSGRenderTextureFormat::RGBA_ASTC_6x5:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_6x5:
         return QRhiTexture::ASTC_6x5;
     case QSSGRenderTextureFormat::RGBA_ASTC_6x6:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_6x6:
         return QRhiTexture::ASTC_6x6;
     case QSSGRenderTextureFormat::RGBA_ASTC_8x5:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_8x5:
         return QRhiTexture::ASTC_8x5;
     case QSSGRenderTextureFormat::RGBA_ASTC_8x6:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_8x6:
         return QRhiTexture::ASTC_8x6;
     case QSSGRenderTextureFormat::RGBA_ASTC_8x8:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_8x8:
         return QRhiTexture::ASTC_8x8;
     case QSSGRenderTextureFormat::RGBA_ASTC_10x5:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_10x5:
         return QRhiTexture::ASTC_10x5;
     case QSSGRenderTextureFormat::RGBA_ASTC_10x6:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_10x6:
         return QRhiTexture::ASTC_10x6;
     case QSSGRenderTextureFormat::RGBA_ASTC_10x8:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_10x8:
         return QRhiTexture::ASTC_10x8;
     case QSSGRenderTextureFormat::RGBA_ASTC_10x10:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_10x10:
         return QRhiTexture::ASTC_10x10;
     case QSSGRenderTextureFormat::RGBA_ASTC_12x10:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_12x10:
         return QRhiTexture::ASTC_12x10;
     case QSSGRenderTextureFormat::RGBA_ASTC_12x12:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_12x12:
         return QRhiTexture::ASTC_12x12;
 
 
@@ -1569,18 +1582,14 @@ void QSSGBufferManager::cleanupUnreferencedBuffers(quint32 frameId, QSSGRenderLa
     // Textures from render extensions
     auto renderExtensionTextureKeyIterator = renderExtensionTexture.cbegin();
     while (renderExtensionTextureKeyIterator != renderExtensionTexture.cend()) {
-        if (isUnused(renderExtensionTextureKeyIterator.value().usageCounts)) {
-            auto rhiTexture = renderExtensionTextureKeyIterator.value().renderImageTexture.m_texture;
-            if (rhiTexture) {
-                if (QSSGBufferManagerStat::enabled(QSSGBufferManagerStat::Level::Debug))
-                   qDebug() << "- releaseTexture: " << (*renderExtensionTextureKeyIterator).renderImageTexture.m_texture << currentLayer;
-                decreaseMemoryStat(rhiTexture);
-                // NOTE: We don't own the texture, it's own by the user, so don't release.
-            }
+        // We do not own the textures, so we just keep track of usage, but
+        // if the texture is no longer valid it means that the extension
+        // has unregistered it, so we can remove it from the map.
+        auto rhiTexture = renderExtensionTextureKeyIterator.value().renderImageTexture.m_texture;
+        if (!rhiTexture)
             renderExtensionTextureKeyIterator = renderExtensionTexture.erase(renderExtensionTextureKeyIterator);
-        } else {
+        else
             ++renderExtensionTextureKeyIterator;
-        }
     }
 
     // Resource Tracking Debug Code
@@ -1888,8 +1897,7 @@ void QSSGBufferManager::registerExtensionResult(const QSSGRenderExtension &exten
         flags.setLinear(!isSRGB);
         const bool isRGBA8 = (texture->format() == QRhiTexture::Format::RGBA8);
         flags.setRgbe8(isRGBA8);
-        const quint32 usageCount = 1 /* At least '1' as long as a texture is registered */;
-        renderExtensionTexture.insert(&extensions, ImageData { QSSGRenderImageTexture{ texture, mipLevels, flags }, {}, usageCount });
+        renderExtensionTexture.insert(&extensions, ImageData { QSSGRenderImageTexture{ texture, mipLevels, flags }, {}, 0 /* version */ });
     } else {
         renderExtensionTexture.insert(&extensions, {});
     }
