@@ -1,31 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Quick 3D.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QSignalSpy>
@@ -59,31 +33,31 @@ void tst_QQuick3DFrustumCamera::testClipAndFov()
     camera.setClipNear(clipNear);
     node = static_cast<QSSGRenderCamera *>(camera.updateSpatialNode(node));
     QCOMPARE(originalNode, node);
-    QVERIFY(node->flags.testFlag(QSSGRenderNode::Flag::CameraDirty));
+    QVERIFY(node->isDirty(QSSGRenderCamera::DirtyFlag::CameraDirty));
     QCOMPARE(clipNear, node->clipNear);
 
     const float clipFar = 0.4f;
     camera.setClipFar(clipFar);
     node = static_cast<QSSGRenderCamera *>(camera.updateSpatialNode(node));
     QCOMPARE(originalNode, node);
-    QVERIFY(node->flags.testFlag(QSSGRenderNode::Flag::CameraDirty));
+    QVERIFY(node->isDirty(QSSGRenderCamera::DirtyFlag::CameraDirty));
     QCOMPARE(clipFar, node->clipFar);
 
     const float fov = 6.2f;
     camera.setFieldOfView(fov);
     node = static_cast<QSSGRenderCamera *>(camera.updateSpatialNode(node));
     QCOMPARE(originalNode, node);
-    QVERIFY(node->flags.testFlag(QSSGRenderNode::Flag::CameraDirty));
+    QVERIFY(node->isDirty(QSSGRenderCamera::DirtyFlag::CameraDirty));
     QCOMPARE(fov, qRadiansToDegrees(node->fov)); // It gets converted inside, so we convert back
 
-    const QQuick3DCamera::FieldOfViewOrientation fovOrientation
-            = QQuick3DCamera::FieldOfViewOrientation::Horizontal;
+    const QQuick3DPerspectiveCamera::FieldOfViewOrientation fovOrientation
+            = QQuick3DPerspectiveCamera::FieldOfViewOrientation::Horizontal;
     camera.setFieldOfViewOrientation(fovOrientation);
     node = static_cast<QSSGRenderCamera *>(camera.updateSpatialNode(node));
     QCOMPARE(originalNode, node);
-    QVERIFY(node->flags.testFlag(QSSGRenderNode::Flag::CameraDirty));
+    QVERIFY(node->isDirty(QSSGRenderCamera::DirtyFlag::CameraDirty));
     QVERIFY(node->fovHorizontal == true);
-    camera.setFieldOfViewOrientation(QQuick3DCamera::FieldOfViewOrientation::Vertical);
+    camera.setFieldOfViewOrientation(QQuick3DPerspectiveCamera::FieldOfViewOrientation::Vertical);
     node = static_cast<QSSGRenderCamera *>(camera.updateSpatialNode(node));
     QVERIFY(node->fovHorizontal == false);
 }
@@ -105,7 +79,7 @@ void tst_QQuick3DFrustumCamera::testFrustum()
     camera.setRight(frustumRight);
     node = static_cast<QSSGRenderCamera *>(camera.updateSpatialNode(node));
     QCOMPARE(originalNode, node);
-    QVERIFY(node->flags.testFlag(QSSGRenderNode::Flag::CameraDirty));
+    QVERIFY(node->isDirty(QSSGRenderCamera::DirtyFlag::CameraDirty));
     QCOMPARE(frustumBottom, node->bottom);
     QCOMPARE(frustumTop, node->top);
     QCOMPARE(frustumLeft, node->left);
