@@ -1,5 +1,5 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QSignalSpy>
 #include <QTest>
@@ -10,6 +10,7 @@
 #include <QtQuick/private/qquicktaphandler_p.h>
 #include <QtQuick3D/private/qquick3ditem2d_p.h>
 #include <QtQuick3D/private/qquick3dnode_p.h>
+#include <QtQml/private/qqmlglobal_p.h>
 
 #include "../shared/util.h"
 
@@ -42,14 +43,22 @@ private:
 
 void tst_Input::initTestCase()
 {
+    // Overrides the internal input handling in QQuick3DViewport so that it's
+    // always true instead of conditionally based on if there are item2Ds in the scene.
+    qputenv("QT_QUICK3D_FORCE_INPUT_HANDLING", "1");
+
     QQuick3DDataTest::initTestCase();
     if (!initialized())
         return;
 
     // Move the system cursor out of the way of test
     // If this isn't possible, don't bother running the test
-    QCursor::setPos(0, 0);
-    if (QCursor::pos() != QPoint(0, 0))
+
+    const auto *screen = QGuiApplication::primaryScreen();
+    const QPoint topLeft = screen ? screen->geometry().topLeft() : QPoint();
+
+    QCursor::setPos(topLeft);
+    if (QCursor::pos() != topLeft)
         QSKIP("It's not possible to move the system cursor, possible test instability");
 }
 
@@ -255,7 +264,7 @@ void tst_Input::fallthrough()
     QSignalSpy pressedChangedSpy(tapHandlerUnderneath, SIGNAL(pressedChanged()));
 
     QTest::mouseClick(view.data(), Qt::LeftButton, Qt::NoModifier, pos);
-    QTRY_COMPARE(pressedChangedSpy.count(), 2);
+    QTRY_COMPARE(pressedChangedSpy.size(), 2);
 
     if (mouseAreaCanBlock) {
         pressedChangedSpy.clear();
@@ -264,8 +273,8 @@ void tst_Input::fallthrough()
         QSignalSpy maPressedChangedSpy(mouseArea, SIGNAL(pressedChanged()));
         mouseArea->setEnabled(true); // accepts the mouse events
         QTest::mouseClick(view.data(), Qt::LeftButton, Qt::NoModifier, pos);
-        QTRY_COMPARE(maPressedChangedSpy.count(), 2);
-        QCOMPARE(pressedChangedSpy.count(), 0);
+        QTRY_COMPARE(maPressedChangedSpy.size(), 2);
+        QCOMPARE(pressedChangedSpy.size(), 0);
     }
 }
 
